function GetMatchingChildItemPath($parentFolder, $itemPattern) {
  return Get-ChildItem $parentFolder | Where-Object { $_.Name -match "$itemPattern" } | Sort-Object | Select-Object -Last 1 | % { $_.FullName }
}

function GetDefaultPackagePaths {
  return @{
    "featureIntegrationPackageXPPath" = GetMatchingChildItemPath "$PSScriptRoot\Resources" "Sitecore Horizon Integration XP [0-9\\.]+ rev. [0-9\\.]{5}\.scwdp\.zip"
    "featureIntegrationPackageXMPath" = GetMatchingChildItemPath "$PSScriptRoot\Resources" "Sitecore Horizon integration XM [0-9\\.]+ rev. [0-9\\.]{5}\.scwdp\.zip"
    "horizon"                         = GetMatchingChildItemPath "$PSScriptRoot\Resources" "HorizonHost [0-9\\.]+ rev. [0-9\\.]{5}\.zip"
  }
}

$packages = GetDefaultPackagePaths

function InstallHorizonHostPackage {
  param (
    [string]$horizonInstanceName,
    [string]$horizonPackagePath = $packages.horizon,
    [string]$horizonAppUrl,
    [string]$sitecoreInstansePath,
    [string]$sitecoreInstanceUrl,
    [string]$sitecoreInstanceInternalUrl = $sitecoreInstanceUrl,
    [string]$horizonPhysicalPath,
    [string]$identityServerUrl,
    [string]$licensePath,
    [bool]$enableContentHub,
    [bool]$enableSXA
  )
  $workDirectory = Join-Path $PSScriptRoot "Configurations"

  # Configure Horizon Host
  $sitecoreAH = @{
    Path                         = Join-Path $workDirectory "\horizon-install-package.json"
    InstanceName                 = $horizonInstanceName
    PhysicalPath                 = $horizonPhysicalPath
    PackagePath                  = $horizonPackagePath
    AppUrl                       = $horizonAppUrl
    ContentManagementUrl         = $sitecoreInstanceUrl
    ContentManagementInternalUrl = $sitecoreInstanceInternalUrl
    LicensePath                  = $licensePath
    IdentityServerUrl            = $identityServerUrl
    EnableContentHub             = $enableContentHub
    EnableSXA                    = $enableSXA
  }

  Write-Host "Running InstallHorizon InstanceName: $($horizonInstanceName)" -ForegroundColor DarkCyan
  Install-SitecoreConfiguration @sitecoreAH -WorkingDirectory $workDirectory
}

function UpdateIdentity {
  param (
    [string]$identityServerPoolName,
    [string]$identityServerPhysicalPath,
    [string]$horizonAppUrl
  )
  $workDirectory = Join-Path $PSScriptRoot "Configurations"
  # Configure Identity
  $sitecoreIS = @{
    Path                       = Join-Path $workDirectory "\horizon-update-identity-server.json"
    HorizonApplicationUrl      = $horizonAppUrl
    IdentityServerPoolName     = $identityServerPoolName
    IdentityServerPhysicalPath = $identityServerPhysicalPath
  }

  Install-SitecoreConfiguration @sitecoreIS -WorkingDirectory $workDirectory
}

function UpdateCM {
  param (
    [string]$sitecoreCmInstanceName,
    [string]$sitecoreInstanceUrl,
    [string]$sitecoreAdminPassword,
    [string]$solrCorePrefix,
    [string]$sitecoreInstansePath,
    [string]$horizonAppUrl,
    [string]$featureIntegrationPackageXPPath = $packages.featureIntegrationPackageXPPath,
    [string]$featureIntegrationPackageXMPath = $packages.featureIntegrationPackageXMPath,
    [string]$topology
  )
  $workDirectory = Join-Path $PSScriptRoot "Configurations"

  # Configure Identity
  $sitecoreCM = @{
    Path                            = Join-Path $workDirectory "\horizon-update-sitecore-instance.json"
    CMSiteName                      = $sitecoreCmInstanceName
    CMSiteUrl                       = $sitecoreInstanceUrl
    CMAdminPassword                 = $sitecoreAdminPassword
    CMSitePhysicalPath              = [System.IO.DirectoryInfo]$sitecoreInstansePath
    featureIntegrationPackageXPPath = $featureIntegrationPackageXPPath
    featureIntegrationPackageXMPath = $featureIntegrationPackageXMPath
    SolrCorePrefix                  = $solrCorePrefix
    HorizonAppUrl                   = $horizonAppUrl
    Topology                        = $topology
  }
  
  Write-Host "Running UpdateCM InstanceName: $($horizonInstanceName)" -ForegroundColor DarkCyan
  Install-SitecoreConfiguration @sitecoreCM -WorkingDirectory $workDirectory
}

function InstallHorizonHost {
  param (
    [string]$horizonInstanceName,
    [string]$horizonPhysicalPath,
    [string]$horizonAppUrl,
    [string]$sitecoreInstanceUrl,
    [string]$sitecoreCmInstanceName,
    [string]$sitecoreInstanceInternalUrl = $sitecoreInstanceUrl,
    [string]$sitecoreAdminPassword,
    [string]$sitecoreInstansePath,
    [string]$identityServerPoolName,
    [string]$identityServerUrl,
    [string]$identityServerPhysicalPath,
    [string]$solrCorePrefix,
    [string]$licensePath,
    [bool]$enableContentHub,
    [bool]$enableSXA,
    [string]$topology
  )

  InstallHorizonHostPackage `
    -horizonInstanceName $horizonInstanceName `
    -horizonPhysicalPath $horizonPhysicalPath `
    -horizonAppUrl $horizonAppUrl `
    -sitecoreInstansePath $sitecoreInstansePath `
    -sitecoreInstanceUrl $sitecoreInstanceUrl `
    -sitecoreInstanceInternalUrl $sitecoreInstanceInternalUrl `
    -identityServerUrl $identityServerUrl `
    -enableContentHub $enableContentHub `
    -enableSXA $enableSXA `
    -licensePath $licensePath

  UpdateIdentity `
    -identityServerPoolName $identityServerPoolName `
    -identityServerPhysicalPath $identityServerPhysicalPath `
    -horizonAppUrl $horizonAppUrl

  UpdateCM `
    -sitecoreCmInstanceName $sitecoreCmInstanceName `
    -sitecoreInstanceUrl $sitecoreInstanceUrl `
    -sitecoreAdminPassword $sitecoreAdminPassword `
    -solrCorePrefix $solrCorePrefix `
    -horizonAppUrl $horizonAppUrl `
    -sitecoreInstansePath $sitecoreInstansePath `
    -topology $topology
}

function Uninstall-HorizonHost {
  param (
    [Parameter(Mandatory = $true)]
    [string]$InstanceName,
    [string]$PhysicalPath = "$env:SystemDrive\inetpub\wwwroot\$InstanceName"
  )

  $workDirectory = Join-Path $PSScriptRoot "Configurations"
  $sitecoreAH = @{
    Path                 = Join-Path $workDirectory "\horizon-install-package.json"
    InstanceName         = $InstanceName
    PhysicalPath         = $PhysicalPath
    # Following parameters aren't needed for uninstall
    PackagePath          = "empty"
    AppUrl               = "empty"
    ContentManagementUrl = "empty"
    LicensePath          = "empty"
    IdentityServerUrl    = "empty"
    EnableContentHub     = $false
    EnableSXA            = $false
  }
  Write-Host "Uninstalling Horizon host: $($InstanceName)" -ForegroundColor DarkCyan
  Uninstall-SitecoreConfiguration @sitecoreAH -WorkingDirectory $workDirectory
}